@@ -68,7 +68,11 @@
  * may be called by another program (such as login), using the -pipe switch.
  */
 
+#include <afsconfig.h>
 #include <afs/param.h>
+
+RCSID("$Header: /cvs/openafs/src/dauth/dlog.c,v 1.5 2001/07/12 19:58:33 shadow Exp $");
+
 #include <afs/stds.h>
 #include <sys/types.h>
 #include <rx/xdr.h>
@@ -89,9 +93,6 @@
 #include <des.h>
 #include <afs/afsutil.h>
 
-#include "../permit_xprt.h"
-
-
 /*
  * The password reading routine in des/readpassword.c will not work if the
  * buffer size passed in is greater than BUFSIZ, so we pretty well have to
