@@ -7,17 +7,19 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef lint
-#endif
 /*******************************************************************\
 * 								    *
 * 	Information Technology Center				    *
 * 	Carnegie-Mellon University				    *
 * 								    *
 \*******************************************************************/
+#include <afsconfig.h>
 #include <afs/param.h>
 
-#ifdef AFS_LINUX20_ENV
+RCSID("$Header: /cvs/openafs/src/lwp/preempt.c,v 1.9 2001/07/12 19:58:48 shadow Exp $");
+
+
+#if defined(AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV) || defined(AFS_DJGPP_ENV)
 int PRE_Block = 0;
 #else
 #include <sys/time.h>
@@ -26,7 +28,7 @@ int PRE_Block = 0;
 #include "lwp.h"
 #include "preempt.h"
 
-#ifdef	AFS_OSF_ENV
+#if defined(AFS_OSF_ENV) || defined(AFS_S390_LINUX20_ENV)
 int PRE_Block = 0;		/* used in lwp.c and process.s */
 #else
 char PRE_Block = 0;		/* used in lwp.c and process.s */
