@@ -15,12 +15,11 @@
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#ifndef lint
-static char sccsid[] = "@(#)utilities.c	5.21 (Berkeley) 2/1/90";
-#endif /* not lint */
-
+#include <afsconfig.h>
 #include <afs/param.h>
 
+RCSID("$Header: /cvs/openafs/src/vfsck/utilities.c,v 1.3 2001/07/12 19:59:28 shadow Exp $");
+
 #include <sys/param.h>
 #define VICE    /* allow us to put our changes in at will */
 #include <sys/time.h>
