@@ -7,21 +7,29 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <afsconfig.h>
 #include <afs/param.h>
+
+RCSID("$Header: /cvs/openafs/src/vlserver/vlserver.c,v 1.11 2001/07/12 19:59:31 shadow Exp $");
+
 #include <afs/stds.h>
 #include <sys/types.h>
 #include <signal.h>
-#ifdef AFS_NT40_ENV
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
+#ifdef AFS_NT40_ENV
+#include <winsock2.h>
 #include <WINNT/afsevent.h>
-#else
+#endif
+#ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
 #include <time.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
 #include <stdio.h>
@@ -45,9 +53,7 @@ int lwps = 9;
 struct vldstats dynamic_statistics;
 struct ubik_dbase *VL_dbase;
 afs_uint32	HostAddress[MAXSERVERID+1];
-extern afs_int32 afsconf_GetKey();
 extern int afsconf_CheckAuth();
-extern int afsconf_ClientAuth();
 extern int afsconf_ServerAuth();
 
 extern afs_int32 ubik_lastYesTime;
@@ -106,7 +112,6 @@ int	argc;
 char	**argv;
 {
     register afs_int32   code;
-    afs_int32	    serverList[MAXSERVERS];
     afs_int32		    myHost;
     struct rx_service	    *tservice;
     struct rx_securityClass *sc[3];
@@ -121,7 +126,8 @@ char	**argv;
     int noAuth = 0, index, i;
     extern int rx_extraPackets;
     char commandLine[150];
-
+    char clones[MAXHOSTSPERCELL];
+ 
 #ifdef	AFS_AIX32_ENV
     /*
      * The following signal action for AIX is necessary so that in case of a 
@@ -164,11 +170,26 @@ char	**argv;
 	    rx_enablePeerRPCStats();
 	} else if (strcmp(argv[index], "-enable_process_stats") == 0) {
 	    rx_enableProcessRPCStats();
+#ifndef AFS_NT40_ENV
+	} else if (strcmp(argv[index], "-syslog")==0) {
+	    /* set syslog logging flag */
+	    serverLogSyslog = 1;
+	} else if (strncmp(argv[index], "-syslog=", 8)==0) {
+	    serverLogSyslog = 1;
+	    serverLogSyslogFacility = atoi(argv[index]+8);
+#endif
 	} else {
 	    /* support help flag */
+#ifndef AFS_NT40_ENV
 	    printf("Usage: vlserver [-p <number of processes>] [-nojumbo] "
-		   /*" [-enable_peer_stats] [-enable_process_stats] " */
+		   "[-syslog[=FACILITY]] "
+		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-help]\n");
+#else
+	    printf("Usage: vlserver [-p <number of processes>] [-nojumbo] "
+		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-help]\n");
+#endif
 	    fflush(stdout);
 	    exit(0);
 	}
@@ -210,19 +231,16 @@ char	**argv;
     }
     bcopy(th->h_addr,&myHost,sizeof(afs_int32));
 
-#if !defined(AFS_HPUX_ENV) && !defined(AFS_NT40_ENV)
+#if !defined(AFS_HPUX_ENV) && !defined(AFS_NT40_ENV) && !defined(AFS_DJGPP_ENV)
     signal(SIGXCPU, CheckSignal_Signal);
 #endif
     /* get list of servers */
-    code = afsconf_GetCellInfo(tdir,(char *)0, AFSCONF_VLDBSERVICE,&info);
+    code = afsconf_GetExtendedCellInfo(tdir,(char *)0, AFSCONF_VLDBSERVICE,
+                                       &info, &clones);
     if (code) {
 	printf("vlserver: Couldn't get cell server list for 'afsvldb'.\n");
 	exit(2);
     }
-    for (index=0,i = 0;index<info.numServers;index++)
-	if (info.hostAddr[index].sin_addr.s_addr != myHost) /* ubik already tacks myHost onto list */
-	    serverList[i++] = info.hostAddr[index].sin_addr.s_addr;
-     serverList[i] = 0;
 
     vldb_confdir = tdir;		/* Preserve our configuration dir */
     /* rxvab no longer supported */
@@ -237,7 +255,8 @@ char	**argv;
     ubik_SRXSecurityRock = (char *) tdir;
     ubik_CheckRXSecurityProc = afsconf_CheckAuth;
     ubik_CheckRXSecurityRock = (char *) tdir;
-    code = ubik_ServerInit(myHost, htons(AFSCONF_VLDBPORT), serverList, vl_dbaseName, &VL_dbase);
+    code = ubik_ServerInitByInfo(myHost, htons(AFSCONF_VLDBPORT), &info,
+				 &clones, vl_dbaseName, &VL_dbase);
     if (code) {
 	printf("vlserver: Ubik init failed with code %d\n",code);
 	exit(2);
